<?php
/**
 * @file
 * Defines the ControllerMenuItemsCollector and utility classes.
 */

require_once dirname(__FILE__) . '/NamingService.php';
require_once dirname(__FILE__) . '/MethodDocParser.php';
require_once dirname(__FILE__) . '/ParserInterface.php';

/**
 * This Collector collects menu items that were defined
 * in Controllers.
 */
class ControllerMenuItemsCollector {
  private $methodDocParser;

  /**
   * Returns lists of menu items defined inside classes that
   * reside in the module directory.
   */
   public function getMenuItemsForModuleDir($module_name, $dir_path) {
    $mask = '{\.php$}si';
    $module_dir_path = drupal_get_path('module', $module_name);
    $controllers_dir_path = trim($dir_path, '\\/');
    $full_dir_path = $module_dir_path . '/' . $controllers_dir_path;
    $options = array(
      'recurse' => FALSE,
    );
    $files = file_scan_directory($full_dir_path, $mask, $options);
    $menu_items = array();
    foreach ($files as $file) {
      require_once $file->uri;
      $controller_name = $file->name;
      $full_class_name = ControllerNamingService::createFullClassName(
        $module_name,
        $controller_name
      );
      if (!class_exists($full_class_name)) {
        $vars = array(
          '%fileName' => $file->filename,
          '%className' => $full_class_name,
        );
        $message = t("The controller class '%className' was not found in the file '%fileName'.", $vars);
        watchdog(CONTROLLER_WATCHDOG_ID, $message, array(), WATCHDOG_ERROR);
        drupal_set_message($message, 'error');
        continue;
      }
      $class_meta = $this->createClassMeta($full_class_name);
      $menu_items = array_merge(
        $menu_items,
        $this->collectMenuItemsFromClass(
          $class_meta,
          $module_name,
          $controller_name,
          $module_dir_path,
          $controllers_dir_path
        )
      );
    }
    return $menu_items;
  }

  /**
   * Returns lists of menu items defined inside classes that reside
   * in the module's controllers/ directory.
   */
  public function getMenuItemsForModule($module_name) {
    return $this->getMenuItemsForModuleDir($module_name, CONTROLLER_DIR_NAME);
  }

  public function setMethodDocParser(ControllerParserInterface $parser) {
    $this->methodDocParser = $parser;
  }

  public function getMethodDocParser() {
    if (NULL === $this->methodDocParser) {
      $this->methodDocParser = new ControllerMethodDocParser();
    }
    return $this->methodDocParser;
  }

  /**
   * Creates the meta object for class.
   */
  protected function createClassMeta($full_class_name) {
    return new ControllerClassMeta($full_class_name);
  }

  /**
   * Collects menu items that defined inside the specifed class.
   */
  protected function collectMenuItemsFromClass(ControllerClassMeta $class_meta, $module_name, $controller_name, $module_dir_path, $controllers_dir_path) {
    $menu_items = array();
    $class_name = $class_meta->getName();
    // We consider only public methods of class as actions.
    $full_class_name = $class_meta->getFullName();
    foreach ($class_meta->getMethods(ReflectionMethod::IS_PUBLIC) as $method_meta) {
      $items = $this->collectMenuItemFromMethod($method_meta, $full_class_name);
      if (!isset($items['path'])) {
        // It is not action method.
        continue;
      }
      $path = $items['path'];
      unset($items['path']);
      // Add the new special item 'action' that should be handled later.
      $items['action'] = $controller_name . CONTROLLER_MENU_SEPARATOR . $method_meta->getName();
      $items['module'] = $module_name;
      $items['file path'] = $module_dir_path;
      $items['file'] = $controllers_dir_path . '/' . $controller_name . '.php';
      $menu_items[$path] = $items;
    }
    return $menu_items;
  }

  /**
   * Allows to collect menu items that defined as php comments.
   */
  protected function collectMenuItemFromMethod(ReflectionMethod $method_meta, $full_class_name) {
    $php_docs = $method_meta->getDocComment();
    $items = $this
      ->getMethodDocParser()
      ->setClassName($full_class_name)
      ->parse($php_docs);
    return $items;
  }
}

/**
 * Used to provide meta for the specified controller class.
 */
class ControllerClassMeta extends ReflectionClass {
  private $fullName;

  public function __construct($full_name) {
    parent::__construct($full_name);
    $this->fullName = $full_name;
  }

  public function getFullName() {
    return $this->fullName;
  }
}
