<?php
/**
 * @file
 * Defines the ControllerMenuItemsNormalizer and utility classes.
 */

require_once dirname(__FILE__) . '/NamingService.php';

/**
 * The ControllerMenuItemsNormalizer do normalization of menu items
 * to follow Drupal's conventions.
 */
class ControllerMenuItemsNormalizer {
  /**
   * Makes normalization of menu items.
   *
   * @return array
   */
  function normalizeMenuItems(array $items) {
    $new_items = array();
    foreach ($items as $path => $item) {
      if (isset($item[CONTROLLER_MENU_ACTION_KEY])) {
        $normalized_items = $this->normalizeMenuItem($item, $path);
        $new_items = array_merge($new_items, $normalized_items);
      }
      else {
        $new_items[$path] = $item;
      }
    }
    return $new_items;
  }

  /**
   * Normalizes single menu item.
   *
   * @return array
   */
  function normalizeMenuItem(array $item_definition, $path) {
    if (isset($item_definition['default'])) {
      $menu_item = new ControllerDefaultMenuItemNormalizer();
    }
    else {
      $menu_item = new ControllerMenuItemNormalizer();
    }
    return $menu_item->normalize($item_definition, $path);
  }
}

/**
 * Normalizes menu item definition to the notation that Drupal uses.
 */
class ControllerMenuItemNormalizer {
  /**
   * Normalizes menu item definition.
   *
   * @return array
   */
  function normalize(array $definition, $path) {
    $this->checkDefinition($definition);

    // 1. Add some standard keys if they missing.
    $definition += $this->getDefaultItems();

    // 2. Handle 'controller' specific items.
    $parts = explode(CONTROLLER_MENU_SEPARATOR, $definition[CONTROLLER_MENU_ACTION_KEY]);
    // If no action method was defined, use 'indexAction'.
    if (empty($parts[1])) {
      $parts[1] = CONTROLLER_DEFAULT_ACTION;
    }
    $controller_name = $parts[0];
    $action_name = $parts[1];
    unset($definition[CONTROLLER_MENU_ACTION_KEY]);

    $module_name = $definition['module'];

    if (!isset($definition['file'])) {
      $definition['file'] = CONTROLLER_DIR_NAME . '/' . $controller_name . '.php';
    }

    $controller_class_name = ControllerNamingService::createFullClassName(
      $module_name,
      $controller_name
    );

    $definition['page callback'] = CONTROLLER_MENU_HANDLER;

    if (!isset($definition['page arguments'])) {
      $definition['page arguments'] = array();
    }

    $definition['page arguments'] = array_merge(
      array(
        $controller_class_name
        . CONTROLLER_MENU_SEPARATOR
        . $action_name
        . CONTROLLER_MENU_SEPARATOR
        . $definition['module'],
      ),
      $definition['page arguments']
    );

    return array($path => $definition);
  }

  /**
   * Returns menu type.
   */
  protected function getType() {
    return MENU_CALLBACK;
  }

  /**
   * Checks menu item definition. Should throw an exception
   * if something is invalid in it.
   */
  protected function checkDefinition(array $definition) {
    $this->checkRequiredKeys($this->getRequiredKeys(), $definition);
  }

  protected function checkRequiredKeys(array $keys, array $definition) {
    foreach ($keys as $key) {
      if (!isset($definition[$key])) {
        throw new RuntimeException("Missing required key '$key' in the menu item definition: " . print_r($definition, 1));
      }
    }
  }

  protected function getRequiredKeys() {
    return array('module', 'action');
  }

  protected function getDefaultItems() {
    return array(
      'type' => $this->getType(),
      'weight' => 0,
      'title' => '',
    );
  }
}

/**
 * Represents extended version of the ControllerMenuItemNormalizer that
 * can handle the 'default' key.
 */
class ControllerDefaultMenuItemNormalizer extends ControllerMenuItemNormalizer {
  /**
   * Checks menu item definition. Should throw an exception
   * if something is invalid in it.
   */
  protected function checkDefinition(array $definition) {
    parent::checkDefinition($definition);
    if (!is_array($definition['default'])) {
      throw new RuntimeException("Invalid format the menu item: " . print_r($definition, 1));
    }
  }

  /**
   * Normalizes menu item definition.
   *
   * @return array
   */
  function normalize(array $definition, $path) {
    $definition += array(
      'type' => MENU_NORMAL_ITEM,
    );
    $parent_definition = parent::normalize($definition, $path);
    $parent_path = $path;

    // Get title and path from the parent item if neccessary.
    $default_item = $parent_definition[$parent_path]['default'];
    $path = $parent_path . '/' . trim(key($default_item), '/');
    $title = reset($default_item);
    unset($parent_definition[$parent_path]['default']);

    $definition = array(
      $path => array_merge(
        $this->getDefaultItems(),
        array(
          'title' => $title,
          'type' => MENU_DEFAULT_LOCAL_TASK,
          'file' => $parent_definition[$parent_path]['file'],
          'module' => $parent_definition[$parent_path]['module'],
        )
      ),
    );

    return array_merge($parent_definition, $definition);
  }
}
